# Databricks notebook source
import pyspark.sql.functions as f
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.express as px
import pandas as pd
import numpy as np

#Import discrepancy index dataset
discrepancy_data=spark.sql("select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual")

#Dual records
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})

discrepancy_data= discrepancy_data.where(( (f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list)) ))

discrepancy_data=discrepancy_data.withColumn('W_E',f.col('DI')*f.col('ExportBToA')).withColumn('W_I',f.col('DI')*f.col('ImportAFromB'))
discrepancy_data = discrepancy_data.withColumn("W", f.when(f.col("ExportBToA") >= f.col("ImportAFromB"), f.col("ExportBToA")).otherwise(f.col("ImportAFromB")) )
discrepancy_data = discrepancy_data.withColumn("WDI", f.col('W')*f.col('DI') )

group1= discrepancy_data.groupby(['NomenCode','Year','Tier']).agg(f.count('Tier').alias('total'),f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , 1).otherwise(0)).alias("matched"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ImportAFromB')).otherwise(0)).alias("matched_import_sum"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ExportBToA')).otherwise(0)).alias("matched_export_sum"),f.sum('ImportAFromB').alias('total_imports'),f.sum('ExportBToA').alias('total_exports'),f.sum('W_E').alias('W_E_sum'),f.sum('W_I').alias('W_I_sum'), f.sum('W').alias('total_W'),f.sum('WDI').alias('total_WDI')).withColumn("Dual recording",f.col('matched')/f.col('total')).withColumn("Export value with dual recording",f.col('matched_export_sum')/f.col('total_exports')).withColumn("Import value with dual recording",f.col('matched_import_sum')/f.col('total_imports')).withColumn('Weighted_di_imports',f.col('W_I_sum')/f.col('total_imports')).withColumn('Weighted_di_exports',f.col('W_E_sum')/f.col('total_exports')).orderBy(['NomenCode','Year','Tier']).withColumn('Weighted DI',f.col('total_WDI')/f.col('total_W'))
group1_pd=group1.toPandas()

group1_matched= discrepancy_data.where((f.col('DI')>-1) & (f.col('DI')<1)).groupby(['NomenCode','Year','Tier']).agg(f.count('Tier').alias('total'),f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , 1).otherwise(0)).alias("matched"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ImportAFromB')).otherwise(0)).alias("matched_import_sum"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ExportBToA')).otherwise(0)).alias("matched_export_sum"),f.sum('ImportAFromB').alias('total_imports'),f.sum('ExportBToA').alias('total_exports'),f.sum('W_E').alias('W_E_sum'),f.sum('W_I').alias('W_I_sum'), f.sum('W').alias('total_W'),f.sum('WDI').alias('total_WDI')).withColumn("Dual recording",f.col('matched')/f.col('total')).withColumn("Export value with dual recording",f.col('matched_export_sum')/f.col('total_exports')).withColumn("Import value with dual recording",f.col('matched_import_sum')/f.col('total_imports')).withColumn('Weighted_di_imports',f.col('W_I_sum')/f.col('total_imports')).withColumn('Weighted_di_exports',f.col('W_E_sum')/f.col('total_exports')).orderBy(['NomenCode','Year','Tier']).withColumn('Weighted DI',f.col('total_WDI')/f.col('total_W'))
group1_matched_pd=group1_matched.toPandas()

viz_1_data=discrepancy_data.where((f.col('Year')==2017) & (f.col('NomenCode')=='H0') & (f.col('Tier')==3) & (f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list))).withColumn('max_value',f.greatest(f.col("ImportAFromB"), f.col("ExportBToA"))).withColumn('max_value_log',f.greatest(f.log(f.col("ImportAFromB")),f.log(f.col('ExportBToA')))).toPandas()

# COMMAND ----------

# DBTITLE 1,Figure 2.A)
num_bins = 20
bin_edges = np.linspace(-0.999, 0.999, num_bins - 1)
bin_edges = np.concatenate(([-1.111], bin_edges, [1.111]))


bin_counts, _ = np.histogram(viz_1_data[viz_1_data['DI'].notnull()]['DI'], bins=bin_edges, density=True)
bin_centers = 0.5 * (bin_edges[1:] + bin_edges[:-1])

bin_width = np.mean(np.diff(bin_centers))


df_histogram = pd.DataFrame({'Discrepancy Index': bin_centers, 'Density': bin_counts * bin_width})
fig = go.Figure(go.Bar(x=df_histogram['Discrepancy Index'], y=df_histogram['Density']*100,
                       marker_color='steelblue',  # Custom color
                       ))  # This adjusts the width of the bars


fig.update_layout(plot_bgcolor='white', bargap=0, width=900, height=600)
fig.update_xaxes(title_text='Discrepancy Index', title_font=dict(size=24), tickfont=dict(size=21), showline=True, linewidth=2, linecolor='black')
fig.update_yaxes(title_text='Frequency', title_font=dict(size=24), tickfont=dict(size=21), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray')

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 2.B)
#Using the same bins as the previous chart
viz_1_data['bin'] = pd.cut(viz_1_data['DI'], bins=bin_edges, include_lowest=True)


bin_sums = viz_1_data.groupby('bin')['max_value'].sum()
total_sum = bin_sums.sum()
bin_percentage = (bin_sums / total_sum) * 100  # Convert to percentage

df_histogram = pd.DataFrame({
    'Discrepancy Index': [(interval.left + interval.right) / 2 for interval in bin_percentage.index],
    'Percentage of max_value': bin_percentage.values
})

fig = go.Figure(go.Bar(x=df_histogram['Discrepancy Index'], y=df_histogram['Percentage of max_value'],
                       marker_color='steelblue'))


fig.update_layout(
    plot_bgcolor='white',
    bargap=0,
    width=900,  
    height=600, 
    xaxis_title="Discrepancy Index",
    yaxis_title="Percentage of Trade Value",
    title_font=dict(size=22),
    xaxis=dict(title_font=dict(size=24), tickfont=dict(size=21), showline=True, linewidth=2, linecolor='black',title_standoff=20),
    yaxis=dict(title_font=dict(size=24), tickfont=dict(size=21), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',title_standoff=20)
)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 3
grouped_df_year_pd = group1_pd.groupby('Year')[['total', 'matched']].sum().reset_index()
grouped_df_year_pd=grouped_df_year_pd[grouped_df_year_pd['Year']<=2022]

grouped_df_year_pd['Dual recording'] = grouped_df_year_pd['matched'] / grouped_df_year_pd['total']

fig=px.line(x=grouped_df_year_pd['Year'],y=grouped_df_year_pd['Dual recording'])
fig['layout']['xaxis']['title']='<b>Year</b>'
fig['layout']['yaxis']['title']='<b>Percentage of successfully mirrored records</b>'
fig.update_layout(
    plot_bgcolor='white',
    xaxis_tickfont=dict(size=14),
    yaxis_tickfont=dict(size=14),
    font=dict(size=16),
    height=500,
    width=950

)
# Update layout
fig.update_layout(
    plot_bgcolor='white',
    xaxis_tickfont=dict(size=22),  
    yaxis_tickfont=dict(size=21),  
    xaxis_title_font=dict(size=23), 
    yaxis_title_font=dict(size=23), 
    font=dict(size=16),
    height=800,
    width=1000,
    yaxis_tickformat=".0%",  
    xaxis_gridcolor='lightgray', 
    yaxis_gridcolor='lightgray',
    xaxis=dict(linecolor='black', linewidth=2,title_standoff=20),  
    yaxis=dict(linecolor='black', linewidth=2,range=[0.1, 0.9],title_standoff=20)  
)


fig.data[0].line.width = 4  
fig.data[0].line.color = "steelblue"  


fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 4
import plotly.graph_objects as go
import numpy as np

bubble_df = group1_pd.groupby(['Year','NomenCode'])[['total', 'matched']].sum().reset_index()
bubble_df=bubble_df[bubble_df['Year']<=2022]

bubble_df['Dual recording'] = bubble_df['matched'] / bubble_df['total']
bubble_df_total=bubble_df.groupby(['Year']).sum('total').reset_index()[['Year','total']]
bubble_df_total

rez_bubble=bubble_df.merge(bubble_df_total,on=['Year'],how='left')
rez_bubble['SIZE']=rez_bubble['total_x']/rez_bubble['total_y']
index=list(range(1962,2023))
columns=['S1','S2','S3','S4','H0','H1','H2','H3','H4','H5']
bubble_data=[]
for i in index:
  bubble_row=[]
  for j in columns:
    if rez_bubble[(rez_bubble['Year']==i) & (rez_bubble['NomenCode']==j)]['SIZE'].empty:
      rez_row_j=0
    else:
      rez_row_j=rez_bubble[(rez_bubble['Year']==i) & (rez_bubble['NomenCode']==j)]['SIZE'].values[0]
    bubble_row.append(rez_row_j)
  bubble_data.append(bubble_row)

#bubble_data preprocessing
nomen_codes = ['S1','S2','S3','S4','H0','H1','H2','H3','H4','H5']
years = list(range(1962,2023))


totals = bubble_data

# Transform matrix to list form for plotting
x_data, y_data, size_data = [], [], []
for i, year in enumerate(years):
    for j, code in enumerate(nomen_codes):
        x_data.append(code)
        y_data.append(year)
        size_data.append(totals[i][j])

# Scale sizes 
def rescale(value, input_range, output_range):
  if value==0:
    return 0
  return output_range[0] + (value - input_range[0]) * (output_range[1] - output_range[0]) / (input_range[1] - input_range[0])

scaled_size = [rescale(size, [0, 1], [13, 26]) for size in size_data]

color_data = ['steelblue' if code.startswith('H') else 'indianred' for code in x_data]

fig = go.Figure(data=go.Scatter(
    x=x_data,
    y=y_data,
    mode='markers',
    marker=dict(
        size=scaled_size,
        sizemode='diameter',
        color=color_data,  # Using the color data
        line=dict(width=0.5, color='black')  # Optional: adding an edge to the marker
    ),
    text=size_data
))

fig.update_layout(
    xaxis_title='<b>Nomenclature</b>',
    yaxis_title='<b>Year</b>',
    xaxis=dict(side='top', showgrid=False),  # x-axis on top and no gridlines
    yaxis=dict(showgrid=False, dtick=1, range=[min(years)-0.5, 2022.5],title_standoff=20),  # no gridlines and showing all y-tick values
    showlegend=False,
    width=1050,
    height=1900,
    plot_bgcolor='white',  # Setting plot background to white
    paper_bgcolor='white',
    xaxis_tickfont=dict(size=23),  # Adjusting the x-axis tick size
    yaxis_tickfont=dict(size=22),  # Adjusting the y-axis tick size
    xaxis_title_font=dict(size=24),  # Adjusting the x-axis label font size
    yaxis_title_font=dict(size=24),  # Adjusting the y-axis label font size
    font=dict(size=16),  # Setting overall background to white
    margin=dict(t=0, b=0, l=0, r=0)
)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 5
line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

fig = make_subplots(rows=1, cols=2, subplot_titles=("<b>The Standard International Trade Classification (SITC Rev.1) </b>"," <b> The Harmonized System (HS 1988) </b>"))


column='Dual recording'
i=0
hs0_data = df[(df['NomenCode'] == 'S1') & (df['Tier'] == '0')]

fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness)),
        row=1, col=1
    )
  
column='Export value with dual recording'
i=1

fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness,dash='dash')),
        row=1, col=1
    )

column='Import value with dual recording'
i=2

fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness,dash='dash')),
        row=1, col=1
    )

# Plot lines for the second subplot (SITC Rev.1)
column='Dual recording'
i=0
hs0_data = df[(df['NomenCode'] == 'H0') & (df['Tier'] == '0')]
fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness),showlegend=False),
        row=1, col=2
    )
column='Export value with dual recording'
i=1
fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness,dash='dash'),showlegend=False),
        row=1, col=2
    )

column='Import value with dual recording'
i=2
fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>', line=dict(color=color_palette[i], width=line_thickness,dash='dash'),showlegend=False),
        row=1, col=2
    )


# Update layout
fig.update_layout(
    title_x=0.5,
    yaxis=dict(range=[0, 1.01],title_standoff=20),
    yaxis2=dict(range=[0, 1.01],title_standoff=20),
    xaxis=dict(title_standoff=20),
    xaxis2=dict(title_standoff=20)
)

#Legend update
fig.update_layout(
    xaxis_title="<b>Year</b>",
    xaxis_title_font=dict(size=24),
    yaxis_title="<b>Percentage</b>",
    yaxis_title_font=dict(size=24),
    xaxis2_title="<b>Year</b>",
    xaxis2_title_font=dict(size=23),
    yaxis2_title="<b>Percentage</b>",
    yaxis2_title_font=dict(size=23),
    yaxis_tickformat='.0%',  
    yaxis2_tickformat='.0%',
    xaxis_tickfont=dict(size=20),
    yaxis_tickfont=dict(size=20),
    xaxis2_tickfont=dict(size=20),
    yaxis2_tickfont=dict(size=20),
    width=1920,  
    height=1080,  
    plot_bgcolor='white',  
    xaxis_gridcolor='lightgray',  
    yaxis_gridcolor='lightgray',  
    xaxis2_gridcolor='lightgray',
    yaxis2_gridcolor='lightgray',
)

fig.update_layout(
    legend=dict(
        orientation="h",  
        xanchor="center", 
        yanchor="bottom", 
        x=0.49,  
        y=-0.22,  
        font=dict(
            size=24  
        )
    ),
)

for i in range(len(fig.layout.annotations)):
    fig.layout.annotations[i].font.size = 23
    fig.layout.annotations[i].y += 0.07


fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 6


line_thickness = 4.6  
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]


fig = make_subplots(rows=1, cols=2, subplot_titles=("<b>The Standard International Trade Classification (SITC Rev.1) </b>"," <b> The Harmonized System (HS 1988) </b>"))


dash_list=['solid', '15px,10px','8px,4px','6px,6px','4px,4px','2px,2px']

# Plot lines for the first subplot (SITC Rev.1)
#["0","1","2","3","4","5"]
names=['Aggregated','Disaggregated','Most disaggregated']
for i, column in enumerate(["0","3","5"]):
  hs0_data = df[(df['NomenCode'] == 'S1') & (df['Tier'] == column)]
  opacity = 1.0 - int(i) * 0.35
  if column=='0':
    trace_name_1='<b> Dual recording - ' +names[i]+'</b>'
    trace_name_2='<b> Import value with dual recording - ' +names[i]+'</b>'
  else:
    trace_name_1='<b>' +names[i]+'</b>'
    trace_name_2='<b>' +names[i]+'</b>'
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Dual recording'], mode='lines', name=trace_name_1, line=dict(color='steelblue', width=line_thickness,dash=dash_list[i]),opacity=opacity),
        row=1, col=1
    )
  """
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Export value with dual recording'], mode='lines', name='<b>'+names[i]+'</b>', line=dict(color=px.colors.sequential.Teal[3], width=line_thickness,dash=dash_list[i]),opacity=opacity),
        row=1, col=1
    )
  """
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Import value with dual recording'], mode='lines', name=trace_name_2, line=dict(color='indianred', width=line_thickness,dash=dash_list[i]),opacity=opacity),
        row=1, col=1
    )
# Plot lines for the second subplot (SITC Rev.1)

#["0","1","2","3"]
for i, column in enumerate(["0","2","3"]):
  hs0_data = df[(df['NomenCode'] == 'H0') & (df['Tier'] == column)]
  opacity = 1.0 - int(i) * 0.35
  
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Dual recording'], mode='lines', name='<b>'+column+'</b>', line=dict(color='steelblue', width=line_thickness,dash=dash_list[i]),opacity=opacity,showlegend=False),
        row=1, col=2
    )
  """
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Export value with dual recording'], mode='lines', name='<b>'+names[i]+'</b>', line=dict(color=px.colors.sequential.Teal[3], width=line_thickness,dash=dash_list[i]),opacity=opacity),
        row=1, col=2
    )
  """
  fig.add_trace(
        go.Scatter(x=hs0_data['Year'], y=hs0_data['Import value with dual recording'], mode='lines', name='<b>'+names[i]+'</b>', line=dict(color='indianred', width=line_thickness,dash=dash_list[i]),opacity=opacity,showlegend=False),
        row=1, col=2
    )



# Update layout
fig.update_layout(
    title_x=0.5,
    yaxis=dict(title_standoff=20),
    yaxis2=dict(title_standoff=20),
    xaxis=dict(title_standoff=20),
    xaxis2=dict(title_standoff=20)
)

#Legend update
fig.update_layout(
    xaxis_title="<b>Year</b>",
    xaxis_title_font=dict(size=23),
    yaxis_title="<b>Percentage</b>",
    yaxis_title_font=dict(size=23),
    xaxis2_title="<b>Year</b>",
    xaxis2_title_font=dict(size=23),
    yaxis2_title="<b>Percentage</b>",
    yaxis2_title_font=dict(size=23),
    yaxis_tickformat='.0%',  
    yaxis2_tickformat='.0%',
    xaxis_tickfont=dict(size=20),
    yaxis_tickfont=dict(size=20),
    xaxis2_tickfont=dict(size=20),
    yaxis2_tickfont=dict(size=20),
    width=1920,  
    height=1080,  
    plot_bgcolor='white',  
    xaxis_gridcolor='lightgray',  
    yaxis_gridcolor='lightgray',  
    xaxis2_gridcolor='lightgray',
    yaxis2_gridcolor='lightgray',
)

fig.update_layout(
    legend=dict(
        orientation="h",  
        xanchor="center",  
        yanchor="bottom",  
        x=0.49,  
        y=-0.22,  
        font=dict(
            size=24  
        )
    ),

)

for i in range(len(fig.layout.annotations)):
    fig.layout.annotations[i].font.size = 23
    fig.layout.annotations[i].y +=0.07


fig.show()

# COMMAND ----------

