# Databricks notebook source
import pyspark.sql.functions as f
from scipy import stats
import numpy as np
import pandas as pd
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.express as px

#Import discrepancy index dataset
discrepancy_data=spark.sql("select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual")

all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})

discrepancy_data= discrepancy_data.where(( (f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list)) ))
discrepancy_data=discrepancy_data.withColumn('W_E',f.col('DI')*f.col('ExportBToA')).withColumn('W_I',f.col('DI')*f.col('ImportAFromB'))
discrepancy_data = discrepancy_data.withColumn("W", f.when(f.col("ExportBToA") >= f.col("ImportAFromB"), f.col("ExportBToA")).otherwise(f.col("ImportAFromB")) )
discrepancy_data = discrepancy_data.withColumn("WDI", f.col('W')*f.col('DI') )

group1= discrepancy_data.groupby(['CountryA','NomenCode','Year','Tier']).agg(f.count('Tier').alias('total'),f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , 1).otherwise(0)).alias("matched"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ImportAFromB')).otherwise(0)).alias("matched_import_sum"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ExportBToA')).otherwise(0)).alias("matched_export_sum"),f.sum('ImportAFromB').alias('total_imports'),f.sum('ExportBToA').alias('total_exports'),f.sum('W_E').alias('W_E_sum'),f.sum('W_I').alias('W_I_sum'), f.sum('W').alias('total_W'),f.sum('WDI').alias('total_WDI')).withColumn("Dual recording",f.col('matched')/f.col('total')).withColumn("Export value with dual recording",f.col('matched_export_sum')/f.col('total_exports')).withColumn("Import value with dual recording",f.col('matched_import_sum')/f.col('total_imports')).withColumn('Weighted_di_imports',f.col('W_I_sum')/f.col('total_imports')).withColumn('Weighted_di_exports',f.col('W_E_sum')/f.col('total_exports')).orderBy(['NomenCode','Year','Tier']).withColumn('Weighted DI',f.col('total_WDI')/f.col('total_W')).toPandas()

columns=['CountryA','Year','Dual recording','Export value with dual recording','Import value with dual recording','Weighted DI']

merged=group1[ (group1['Tier']=='0') & (group1['NomenCode']=='H0')][columns].merge(group1[ (group1['Tier']=='3') & (group1['NomenCode']=='H0')][columns],on=['CountryA','Year'])
merged['agg_loss']=1-merged['Dual recording_y']/merged['Dual recording_x']
merged['agg_loss_value_import']=1-merged['Import value with dual recording_y']/merged['Import value with dual recording_x']
merged['agg_loss_value_export']=1-merged['Export value with dual recording_x']/merged['Export value with dual recording_x']

merged_sitc=group1[ (group1['Tier']=='0') & (group1['NomenCode']=='S1')][columns].merge(group1[ (group1['Tier']=='3') & (group1['NomenCode']=='H0')][columns],on=['CountryA','Year'])
merged_sitc['agg_loss']=1-merged_sitc['Dual recording_y']/merged_sitc['Dual recording_x']
merged_sitc['agg_loss_value_import']=1-merged_sitc['Import value with dual recording_y']/merged_sitc['Import value with dual recording_x']
merged_sitc['agg_loss_value_export']=1-merged_sitc['Export value with dual recording_x']/merged_sitc['Export value with dual recording_x']


# COMMAND ----------

# DBTITLE 1,Figure 7

def calculate_confidence_interval(x, confidence=0.95):
    """Calculate the confidence interval for a given sample array `x`."""
    n = len(x)
    m, se = np.mean(x), stats.sem(x)
    h = se * stats.t.ppf((1 + confidence) / 2., n-1)
    return m, m-h, m+h

merged.replace([np.inf, -np.inf], np.nan, inplace=True)
merged_conf=merged[merged['agg_loss'].notnull()]
grouped_conf = merged_conf.groupby('Year')['agg_loss'].apply(lambda x: calculate_confidence_interval(x)).reset_index()

# Expand the result into separate columns
grouped_conf[['Mean', 'CI_Lower', 'CI_Upper']] = pd.DataFrame(grouped_conf['agg_loss'].tolist(), index=grouped_conf.index)

# Drop the original column with tuple results
grouped_conf.drop(columns=['agg_loss'], inplace=True)

grouped_conf=grouped_conf[(grouped_conf['Year']>2000) & (grouped_conf['Year']<2020)]


#SITC
merged_sitc.replace([np.inf, -np.inf], np.nan, inplace=True)
merged_conf_sitc=merged_sitc[merged_sitc['agg_loss'].notnull()]
grouped_conf_sitc = merged_conf_sitc.groupby('Year')['agg_loss'].apply(lambda x: calculate_confidence_interval(x)).reset_index()

# Expand the result into separate columns
grouped_conf_sitc[['Mean', 'CI_Lower', 'CI_Upper']] = pd.DataFrame(grouped_conf_sitc['agg_loss'].tolist(), index=grouped_conf_sitc.index)

# Drop the original column with tuple results
grouped_conf_sitc.drop(columns=['agg_loss'], inplace=True)

grouped_conf_sitc=grouped_conf_sitc[(grouped_conf_sitc['Year']>1990) & (grouped_conf_sitc['Year']<2020)]

fig = make_subplots(rows=1, cols=2, subplot_titles=("<b>The Standard International Trade Classification (SITC Rev.1) </b>"," <b> The Harmonized System (HS 1988) </b>"))
fig.add_trace(go.Scatter(x=grouped_conf_sitc['Year'], y=grouped_conf_sitc['Mean'], mode='lines', name='Mean', line=dict(color='steelblue'),showlegend=True,legendrank=1),row=1,col=1)
fig.add_trace(go.Scatter(x=grouped_conf_sitc['Year'], y=grouped_conf_sitc['CI_Upper'], mode='lines', name='Upper Bound', showlegend=False,
               line=dict(width=0)),row=1,col=1)
fig.add_trace(go.Scatter(x=grouped_conf_sitc['Year'], y=grouped_conf_sitc['CI_Lower'], mode='lines', name='95% Confidence Interval',
               fill='tonexty', fillcolor='rgba(0,100,80,0.2)', line=dict(width=0), showlegend=True,legendrank=2),row=1,col=1)

fig.add_trace( go.Scatter(x=grouped_conf['Year'], y=grouped_conf['Mean'], mode='lines', name='Mean', line=dict(color='steelblue'),showlegend=False),row=1,col=2)
fig.add_trace( go.Scatter(x=grouped_conf['Year'], y=grouped_conf['CI_Upper'], mode='lines', name='Upper Bound', showlegend=False,
               line=dict(width=0)),row=1,col=2)
fig.add_trace( go.Scatter(x=grouped_conf['Year'], y=grouped_conf['CI_Lower'], mode='lines', name='95% Confidence Interval',
               fill='tonexty', fillcolor='rgba(0,100,80,0.2)', line=dict(width=0), showlegend=False),row=1,col=2)


fig.update_layout(
    title_x=0.5,
    yaxis=dict(range=[0, 1.01]),
    yaxis2=dict(range=[0, 1.01]),
)
"""
# Update layout
fig.update_layout(
    title_x=0.5,
    yaxis=dict(range=[0, 1.01],title_standoff=20),
    yaxis2=dict(range=[0, 1.01],title_standoff=20),
    xaxis=dict(title_standoff=20),
    xaxis2=dict(title_standoff=20)
)
"""

#Legend update
fig.update_layout(
    xaxis_title="<b>Year</b>",
    xaxis_title_font=dict(size=23),
    yaxis_title="<b>Average loss of successful mirroring on granular data</b>",
    yaxis_title_font=dict(size=23),
    xaxis2_title="<b>Year</b>",
    xaxis2_title_font=dict(size=23),
    yaxis2_title="<b>Average loss of successful mirroring on granular data</b>",
    yaxis2_title_font=dict(size=23),
    yaxis_tickformat='.0%',  
    yaxis2_tickformat='.0%',
    xaxis_tickfont=dict(size=20),
    yaxis_tickfont=dict(size=20),
    xaxis2_tickfont=dict(size=20),
    yaxis2_tickfont=dict(size=20),
    width=1720,  
    height=1080, 
    plot_bgcolor='white',  
    xaxis_gridcolor='lightgray',  
    yaxis_gridcolor='lightgray',  
    xaxis2_gridcolor='lightgray',
    yaxis2_gridcolor='lightgray'
    
)

fig.update_layout(
    legend=dict(
        orientation="h",  # Set legend orientation to horizontal
        xanchor="center",  # Anchor the legend at the center
        yanchor="bottom",  # Anchor the legend below the plot
        x=0.49,  # Position the legend in the middle of the x-axis
        y=-0.22,  # Position the legend below the plot
        font=dict(
            size=24  # Set the font size here
        )
    ),
)
# Increase the subtitle text size by modifying annotations
for i in range(len(fig.layout.annotations)):
    fig.layout.annotations[i].font.size = 23
    fig.layout.annotations[i].y +=0.07

# Display the plot
fig.show()

# COMMAND ----------

