# Databricks notebook source
import pyspark.sql.functions as f
import pandas as pd
import plotly.graph_objects as go
import numpy as np
import plotly.express as px


COUNTRY_A='MDG'
COUNTRY_B='FRA'

discrepancy_data=spark.sql("select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual")
discrepancy_data=discrepancy_data.where( (f.col('NomenCode')=='H0') & (f.col('Year')<2022))

#Country list
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})


#Region data
region_data=spark.sql("select * from prd_datascience_dianapublic.diana.countryregion")
region_data_df=region_data.toPandas()


#Existing diana database with ProductCode and ProductDescription on all NomenclatureCode(s) and Tier(s)
code_data=spark.sql('select * from prd_datascience_dianapublic.diana.productlist')
#Broad groups for HS classification
product_group_hs_data=spark.sql('select * from prd_datascience_dianapublic.diana.ProductGroups_hs')
commodity_names=spark.sql("select * from prd_datascience_dianapublic.default.commodity_names_hs_6")

"""
1. Total trade 
"""
#MDG Reported Exports to FRA and FRA reported importds from MDG
balance_COUNTRY_A=discrepancy_data.where((f.col('CountryA')==COUNTRY_B) & (f.col('Tier')==0) & (f.col('CountryB')==COUNTRY_A)).select(f.col('Year').alias('Year_MDG'),f.col('ExportBtoA').alias('ExportBtoA_MDG'),f.col('ImportAFromB').alias('ImportAFromB_MDG'),f.col('DI').alias('DI_MDG'))

#FRA Reported Exports to MDG and MDG reported imports from FRA
balance_COUNTRY_B=discrepancy_data.where((f.col('CountryA')==COUNTRY_A) & (f.col('Tier')==0) & (f.col('CountryB')==COUNTRY_B)).select(f.col('Year').alias('Year_FRA'),f.col('ExportBtoA').alias('ExportBtoA_FRA'),f.col('ImportAFromB').alias('ImportAFromB_FRA'),f.col('DI').alias('DI_FRA'))

balances_data=balance_COUNTRY_A.join(balance_COUNTRY_B,[f.col('Year_MDG')==f.col('Year_FRA')]).withColumn('TradeBalance_MDG_reported',f.col('ExportBtoA_MDG')-f.col('ImportAFromB_FRA')).withColumn('TradeBalance_FRA_reported',f.col('ExportBtoA_FRA')-f.col('ImportAFromB_MDG')).select(f.col('Year_MDG').alias('Year'),'TradeBalance_MDG_reported','TradeBalance_FRA_reported','DI_MDG','DI_FRA')
balances_data_pd=balances_data.toPandas()
balances_data_pd

# COMMAND ----------

# DBTITLE 1,Figure 8

# Create traces
balances_data_pd=balances_data_pd[balances_data_pd['Year']>=2000]
fig = go.Figure()
fig.add_trace(go.Scatter(x=balances_data_pd['Year'], y=balances_data_pd['TradeBalance_MDG_reported']/1000,
                    mode='lines',line=dict(
                    color='royalblue',
                    width=3
                ),
                    name='as reported by Madagascar'))

fig.add_trace(go.Scatter(x=balances_data_pd['Year'], y=abs(balances_data_pd['TradeBalance_FRA_reported'])/1000,
                    mode='lines',line=dict(
                    color='indianred',
                    width=3
                ),
                    name='as reported by France'))



fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')

fig['layout']['xaxis']['title']='Year'
fig['layout']['yaxis']['title']='Million USD$'
#title=dict(text="Madagascar's annual trade surplus with France",yref='paper'),
fig.update_layout(
    plot_bgcolor='white',
    width=1200, height=600, legend=dict( font=dict(size=15))
)
fig.update_xaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 8 1.A)
balance_COUNTRY_A_pd=balance_COUNTRY_A.select(f.col('Year_MDG').alias('Year'),f.col('ExportBtoA_MDG').alias('Export_MDG_to_FRA_MDG_reported'),f.col('ImportAFromB_MDG').alias('Export_MDG_to_FRA_FRA_reported')).orderBy('Year').toPandas()
balance_COUNTRY_A_pd=balance_COUNTRY_A_pd[balance_COUNTRY_A_pd['Year']>=2000]

fig = go.Figure()
fig.add_trace(go.Scatter(x=balance_COUNTRY_A_pd['Year'], y=balance_COUNTRY_A_pd['Export_MDG_to_FRA_MDG_reported']/1000,
                    mode='lines',line=dict(
                    color='royalblue',
                    width=3
                ),
                    name='as reported by Madagascar'))

fig.add_trace(go.Scatter(x=balance_COUNTRY_A_pd['Year'], y=abs(balance_COUNTRY_A_pd['Export_MDG_to_FRA_FRA_reported'])/1000,
                    mode='lines',line=dict(
                    color='indianred',
                    width=3
                ),
                    name='as reported by France'))



fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')

fig['layout']['xaxis']['title']='Year'
fig['layout']['yaxis']['title']='Million USD$'
fig.update_layout(
    plot_bgcolor='white',
    width=1200, height=600, legend=dict( font=dict(size=15))
)
fig.update_xaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 8. 1.B)
HIST_data_year_pd_MDG=discrepancy_data.where((f.col('CountryA')==COUNTRY_B) & (f.col('Year')==2015) & (f.col('Tier')==3) & (f.col('CountryB')==COUNTRY_A)).withColumn('TradeValue',f.when(f.col('ImportAfromB')>f.col('ExportBToA'),f.col('ImportAfromB')).otherwise(f.col('ExportBToA'))).select(f.col('Year'),f.col('ProductCode'),f.col('ExportBtoA').alias('Export_MDG_To_FRA'),f.col('ImportAFromB').alias('Import_FRA_from_MDG'),f.col('TradeValue'),f.col('DI')).toPandas()

fig=px.histogram(HIST_data_year_pd_MDG,x='DI',y='TradeValue',histfunc='sum',labels={
                     "Discrepancy Index",
                     "Trade Value"
                 })
fig.update_layout(yaxis_title='Trade Value',xaxis_title='Discrepancy Index')
fig.update_layout(
    plot_bgcolor='white',
    width=900, height=600,bargap=0.05
)
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)        

# COMMAND ----------

# DBTITLE 1,Figure 8 2.A)
balance_COUNTRY_B_pd=balance_COUNTRY_B.select(f.col('Year_FRA').alias('Year'),f.col('ExportBtoA_FRA').alias('Export_FRA_to_MDG_FRA_reported'),f.col('ImportAFromB_FRA').alias('Export_FRA_to_MDG_MDG_reported')).orderBy('Year').toPandas()

# Create traces
balance_COUNTRY_B_pd=balance_COUNTRY_B_pd[balance_COUNTRY_B_pd['Year']>=2000]
fig = go.Figure()
fig.add_trace(go.Scatter(x=balance_COUNTRY_B_pd['Year'], y=balance_COUNTRY_B_pd['Export_FRA_to_MDG_MDG_reported']/1000,
                    mode='lines',line=dict(
                    color='royalblue',
                    width=3
                ),
                    name='as reported by Madagascar'))

fig.add_trace(go.Scatter(x=balance_COUNTRY_B_pd['Year'], y=abs(balance_COUNTRY_B_pd['Export_FRA_to_MDG_FRA_reported'])/1000,
                    mode='lines',line=dict(
                    color='indianred',
                    width=3
                ),
                    name='as reported by France'))



fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')

fig['layout']['xaxis']['title']='Year'
fig['layout']['yaxis']['title']='Million USD$'
#title=dict(text="Exports from France to Madagascar",yref='paper'),
fig.update_layout(
    plot_bgcolor='white',
    width=1200, height=600, legend=dict( font=dict(size=15))
)
fig.update_xaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=20,title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 8. 2.B)
HIST_data_year_pd=discrepancy_data.where((f.col('CountryA')==COUNTRY_A) & (f.col('Year')==2015) & (f.col('Tier')==3) & (f.col('CountryB')==COUNTRY_B)).withColumn('TradeValue',f.when(f.col('ImportAfromB')>f.col('ExportBToA'),f.col('ImportAfromB')).otherwise(f.col('ExportBToA'))).select(f.col('Year'),f.col('ProductCode'),f.col('ExportBtoA').alias('Export_FRA_To_MDG'),f.col('ImportAFromB').alias('Import_MDG_from_FRA'),f.col('TradeValue'),f.col('DI')).toPandas()

fig=px.histogram(HIST_data_year_pd,x='DI',y='TradeValue',histfunc='sum')
fig.update_layout(yaxis_title='Trade Value',xaxis_title='Discrepancy Index')
fig.update_layout(
    plot_bgcolor='white',
    width=900, height=600,bargap=0.05
)
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',title_font=dict(size=24), tickfont=dict(size=18),title_standoff=20)  

# COMMAND ----------

# DBTITLE 1,Figure 22
COUNTRY='FRA'
REF_YEAR=2015

import warnings
warnings.filterwarnings('ignore')

TRADE_FLOW=2
TIER=0

#Export partner data for countries
export_partner_composition_data=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==0) & (f.col('CountryA').isin(country_list)))
export_partner_composition_data_pd=export_partner_composition_data.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Merge so region data
export_partner_composition_data_pd1=export_partner_composition_data_pd.rename(columns={'CountryA':'Country','ExportBToA':'TradeValue'})
export_partner_composition_data_pd1=export_partner_composition_data_pd1.merge(region_data_df,left_on='Country',right_on='ISO3',how='left')[['Country','TradeValue','RegionName']]
export_partner_composition_data_pd1=export_partner_composition_data_pd1.dropna(how='any')
#2.Continent total trade
export_partner_continent=export_partner_composition_data_pd1.groupby(['RegionName'],as_index=False)[['TradeValue']].sum()
export_partner_continent['Country']=export_partner_continent['RegionName']
export_partner_continent['RegionName']='world'
#3.World total trade
export_partner_world=pd.DataFrame([export_partner_continent['TradeValue'].sum()],columns=['TradeValue'])
export_partner_world['Country']='world'
export_partner_world['RegionName']=""
#4.Append all dataframes
export_partner_composition_data_pd1=export_partner_composition_data_pd1.append(export_partner_continent[['Country','TradeValue','RegionName']]).append(export_partner_world[['Country','TradeValue','RegionName']])

#Deriving discrepancy indicators for each partner (Indicators are currently derived from a Tier 3 level)
median_di_data=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==3)).groupBy('CountryA').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd=median_di_data.toPandas()
median_di_data_pd=median_di_data_pd.rename(columns={'CountryA':'Country'})

viz_data=export_partner_composition_data_pd1.merge(median_di_data_pd,on='Country',how='left')
viz_data['TradeValue']=viz_data['TradeValue']/1000

fig = go.Figure(go.Treemap(labels=viz_data['Country'], values=viz_data['TradeValue'], parents=viz_data['RegionName'],customdata=viz_data['CIF_FOB_ratio'],textinfo = "label+percent entry" ,branchvalues='total',pathbar_textfont_size=10,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Overreporting export', '1','Underreporting export','2']
        ),cmax=2,cmin=0,cmid=1),hovertemplate='<b>Export destination: %{label}</b> <br><b>Region:</b> %{parent}<br><b>Export:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata}<extra></extra>'))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#uniformtext=dict(minsize=10,mode='hide'),
title_text='France export destinations (2015) and their CIF/FOB ratio'
fig.update_layout(
  
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text='France export destinations (2015) and their CIF/FOB ratio',font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 23
import warnings
warnings.filterwarnings('ignore')

COUNTRY='MDG'
REF_YEAR=2015
TRADE_FLOW=1
TIER=0


#Export partner data for countries
export_partner_composition_data=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==0) & (f.col('CountryB').isin(country_list))).withColumn('Import',f.when(f.col('ImportAfromB')>f.col('ExportBToA'),f.col('ImportAfromB')).otherwise(f.col('ExportBToA')))
export_partner_composition_data_pd=export_partner_composition_data.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Merge so region data
export_partner_composition_data_pd1=export_partner_composition_data_pd.rename(columns={'CountryB':'Country','Import':'TradeValue'})
export_partner_composition_data_pd1=export_partner_composition_data_pd1.merge(region_data_df,left_on='Country',right_on='ISO3',how='left')[['Country','TradeValue','RegionName']]
export_partner_composition_data_pd1=export_partner_composition_data_pd1.dropna(how='any')
#2.Continent total trade
export_partner_continent=export_partner_composition_data_pd1.groupby(['RegionName'],as_index=False)[['TradeValue']].sum()
export_partner_continent['Country']=export_partner_continent['RegionName']
export_partner_continent['RegionName']='world'
#3.World total trade
export_partner_world=pd.DataFrame([export_partner_continent['TradeValue'].sum()],columns=['TradeValue'])
export_partner_world['Country']='world'
export_partner_world['RegionName']=""
#4.Append all dataframes
export_partner_composition_data_pd1=export_partner_composition_data_pd1.append(export_partner_continent[['Country','TradeValue','RegionName']]).append(export_partner_world[['Country','TradeValue','RegionName']])

#Deriving discrepancy indicators for each partner (Indicators are currently derived from a Tier 3 level)
median_di_data=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==3)).groupBy('CountryB').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd=median_di_data.toPandas()
median_di_data_pd=median_di_data_pd.rename(columns={'CountryB':'Country'})

viz_data=export_partner_composition_data_pd1.merge(median_di_data_pd,on='Country',how='left')
viz_data['TradeValue']=viz_data['TradeValue']/1000

fig = go.Figure(go.Treemap(labels=viz_data['Country'], values=viz_data['TradeValue'], parents=viz_data['RegionName'],customdata=viz_data['CIF_FOB_ratio'],textinfo = "label+percent entry" ,branchvalues='total',pathbar_textfont_size=10,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Underreporting import', '1','Overreporting import','2']
        ),cmax=2,cmin=0,cmid=1),hovertemplate='<b>Import origin: %{label}</b> <br><b>Region:</b> %{parent}<br><b>Import:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata}<extra></extra>'))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#uniformtext=dict(minsize=10,mode='hide'),
text_title='Madagascar import origins (2015) and their CIF/FOB ratio'
fig.update_layout(
  
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text=text_title,font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()