# Databricks notebook source
import pyspark.sql.functions as f
import pandas as pd
import plotly.graph_objects as go


discrepancy_data=spark.sql("select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual")
discrepancy_data=discrepancy_data.where( (f.col('NomenCode')=='H0') & (f.col('Year')<2022))

#Country list
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})

#HS6 Commodity names
commodity_names=spark.sql("select * from prd_datascience_dianapublic.default.commodity_names_hs_6")
#Region data
region_data=spark.sql("select * from prd_datascience_dianapublic.diana.countryregion")
region_data_df=region_data.toPandas()


#Existing diana database with ProductCode and ProductDescription on all NomenclatureCode(s) and Tier(s)
code_data=spark.sql('select * from prd_datascience_dianapublic.diana.productlist')
#Broad groups for HS classification
product_group_hs_data=spark.sql('select * from prd_datascience_dianapublic.diana.ProductGroups_hs')

#Trade reporting indicators
trade_reporting_indicators=spark.sql('select * from prd_datascience_dianapublic.dianapublic.country_trade_reporting_indicators')
trade_reporting_indicators=trade_reporting_indicators.where((f.col('NomenCode')=='H0') & (f.col('flow_type')=='import_reporting') & (f.col('Year')<2022))
trade_reporting_indicators_pd=trade_reporting_indicators.toPandas()

COUNTRY='SEN'
REF_YEAR=2021

import_reporting=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('CountryB').isin(country_list)) & (f.col('Tier')==0)).groupby('Year').agg(f.sum('ImportAFromB').alias('Import-SEN-reported'),f.sum('ExportBToA').alias('Import-Partner-reported'))
import_reporting=import_reporting.toPandas()

export_reporting=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Tier')==0) & (f.col('CountryA').isin(country_list))).groupby('Year').agg(f.sum('ImportAFromB').alias('Export-Partner-reported'),f.sum('ExportBToA').alias('Export-SEN-reported'))
export_reporting=export_reporting.toPandas()



# COMMAND ----------

# DBTITLE 1,Figure 12
fig = go.Figure()
import_reporting=import_reporting.sort_values('Year')
name_1='As reported by '+COUNTRY
fig.add_trace(go.Scatter(x=import_reporting['Year'], y=import_reporting['Import-Partner-reported']/1000000,
                    mode='lines',line=dict(
                    color='royalblue', dash='dot',
                    width=3
                ),
                    name='As reported by partners'))

fig.add_trace(go.Scatter(x=import_reporting['Year'], y=import_reporting['Import-SEN-reported']/1000000,
                    mode='lines',line=dict(
                    color='indianred',
                    width=3
                ),
                    name=name_1))



fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')

fig['layout']['xaxis']['title']='Year'
fig['layout']['yaxis']['title']='Billion USD$'
#title=dict(text="Madagascar's annual trade surplus with France",yref='paper'),
fig.update_layout(
    plot_bgcolor='white',
    xaxis_tickfont=dict(size=14),
    yaxis_tickfont=dict(size=14),
    font=dict(size=16),
    height=500,
    width=950

)
#,nticks=20
fig.update_xaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=10,title_standoff=20)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=10,title_standoff=20)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 13
COUNTRY = 'SEN'
indicators = ['cif_fob_ratio', 'median_DI']
country_data = trade_reporting_indicators_pd[trade_reporting_indicators_pd['Country'] == COUNTRY].sort_values(by='Year', ascending=True)

# Initialize the figure
fig = go.Figure()
indicator_name=['CIF/FOB Ratio','Median DI']

# Colors and styles based on the second plot's appearance
line_colors = ['rgba(31, 119, 180, 0.6)', 'rgba(205, 92, 92, 0.7)']  # Assigning FOB and CIF color equivalents
line_styles = ['dash', 'dot']  # Assigning line styles
marker_symbols = ['circle', 'square']  # Assigning marker symbols

# Add traces with the specified appearance
for i, indicator in enumerate(indicators):
    fig.add_trace(go.Scatter(x=country_data['Year'], y=country_data[indicator],
                             mode='lines+markers',
                             name=indicator_name[i],
                             line=dict(color=line_colors[i], dash=line_styles[i],width=4),
                             marker=dict(symbol=marker_symbols[i], size=8)))

title = ""
fig.update_layout(
    width=1600,
    height=600,
    title=dict(text=title, font=dict(size=25)),
    plot_bgcolor='white',
    font=dict(size=20),
    legend=dict(font=dict(size=22), orientation='h', xanchor='center', x=0.5, y=1.1),
    xaxis=dict(tickfont=dict(size=20), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray', nticks=10),
    yaxis=dict(tickfont=dict(size=20), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray', nticks=10)
)

# Update legend to only show for the first two traces (for clarity, based on the original request)
fig.update_traces(showlegend=True)
for trace in fig.data[2:]:
    trace.showlegend = False

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 14
fig = go.Figure()
export_reporting=export_reporting.sort_values('Year')
name_1='As reported by '+COUNTRY
fig.add_trace(go.Scatter(x=export_reporting['Year'], y=export_reporting['Export-Partner-reported'],
                    mode='lines',line=dict(
                    color='royalblue',
                    width=3,dash='dot'
                ),
                    name='As reported by partners'))

fig.add_trace(go.Scatter(x=export_reporting['Year'], y=export_reporting['Export-SEN-reported'],
                    mode='lines',line=dict(
                    color='indianred',
                    width=3
                ),
                    name=name_1))



fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')

fig['layout']['xaxis']['title']='Year'
fig['layout']['yaxis']['title']='Billion USD$'
#title=dict(text="Madagascar's annual trade surplus with France",yref='paper'),
fig.update_layout(
    plot_bgcolor='white',
    xaxis_tickfont=dict(size=14),
    yaxis_tickfont=dict(size=14),
    font=dict(size=16),
    height=500,
    width=950

)
#,nticks=20
fig.update_xaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=10)
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',nticks=10)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 24
import warnings
warnings.filterwarnings('ignore')

TIER=3
TRADE_FLOW=2

code_data=code_data.where((f.col('NomenclatureCode')=='HS') & (f.col('Tier')==TIER) )


#Merging ProductCode,productDescription data with ProductCode,ProductGroup data
code_data=code_data.select('ProductCode','ProductDescription').withColumn('ProductCodeL',f.substring('ProductCode',1,2)).join(product_group_hs_data.withColumn('ProductCodeR',f.col('ProductCode')).drop('ProductCode'),[f.col('ProductCodeL')==f.col('ProductCodeR')],how='full').select('ProductCode','ProductDescription','ProductGroup')

#Merging export composition data of country with the productDescription and ProductGroup
#ProductCode,Export,ProductDescription,ProductGroup
export_composition_data=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==TIER) & (f.col('CountryA')=='WLD')).withColumn('Export',f.col('ExportBToA')).join(code_data,'ProductCode',how='left').select('ProductCode','Export','ProductDescription','ProductGroup')

#Product Indicators data (used for coloring the map)
#median_di_data_1=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==TIER) & (f.col('CountryA').isin(country_list)) & (f.length(f.col('ProductCode'))==2)).groupBy('ProductCode').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_1=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==TIER) & (f.col('CountryA').isin(country_list)) & (f.col('Tier')==TIER)).groupBy('ProductCode').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd_1=median_di_data_1.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Convert to pandas
export_composition_data_pd1=export_composition_data.toPandas()
#2.ProductGroup total trade
export_group=export_composition_data_pd1.groupby(['ProductGroup'],as_index=False)[['Export']].sum()
export_group['ProductCode']=export_group['ProductGroup']
export_group['ProductGroup']='All Commodities'
export_group['ProductDescription']=''
#3.All commodities total trade
export_group_all=pd.DataFrame([export_group['Export'].sum()],columns=['Export'])
export_group_all['ProductCode']='All Commodities'
export_group_all['ProductGroup']=""
export_group_all['ProductDescription']=''
#4.Append all dataframes
export_composition_data_pd1=export_composition_data_pd1.append(export_group[['ProductCode','Export','ProductDescription','ProductGroup']]).append(export_group_all[['ProductCode','Export','ProductDescription','ProductGroup']])
#export_composition_data_pd1


viz_data=export_composition_data_pd1.merge(median_di_data_pd_1,on='ProductCode',how='left')
commodity_names_pd=commodity_names.toPandas()
commodity_names_pd['CommodityCode']=commodity_names_pd['HS6 ID'].astype(str).str[-6:]
viz_data=viz_data.merge(commodity_names_pd[['CommodityCode','HS6']],left_on='ProductCode',right_on='CommodityCode',how='left').drop(columns={'CommodityCode'})
viz_data=viz_data.drop(columns='ProductDescription').rename(columns={'HS6':'ProductDescription'})
viz_data['Export']=viz_data['Export']/1000

fig = go.Figure(go.Treemap(labels=viz_data['ProductCode'], values=viz_data['Export'], parents=viz_data['ProductGroup'],customdata = viz_data[['ProductDescription','CIF_FOB_ratio']],textinfo = "label+percent entry", branchvalues='total',pathbar_textfont_size=15,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Negative transporation costs', '1','Positive transportation costs','2']
        ),cmin=0,cmax=2,cmid=1),hovertemplate='<b>Commodity Code: %{label}</b> <br><b>Commodity Description:</b>%{customdata[0]}<br><b>Commodity Group:</b> %{parent}<br><b>Export:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata[1]}<extra></extra>' ))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#hovertext=viz_data['CIF/FOB']  
#<br><b>X</b>: %{x}<br>'
text_title=COUNTRY+'\'s exports and their CIF/FOB ratio'

fig.update_layout(
uniformtext=dict(minsize=10,mode='hide'),
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text=text_title,font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 25
TRADE_FLOW=2
TIER=0



#Export partner data for countries
export_partner_composition_data=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==0) & (f.col('CountryA').isin(country_list)))
export_partner_composition_data_pd=export_partner_composition_data.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Merge so region data
export_partner_composition_data_pd1=export_partner_composition_data_pd.rename(columns={'CountryA':'Country','ExportBToA':'TradeValue'})
export_partner_composition_data_pd1=export_partner_composition_data_pd1.merge(region_data_df,left_on='Country',right_on='ISO3',how='left')[['Country','TradeValue','RegionName']]
export_partner_composition_data_pd1=export_partner_composition_data_pd1.dropna(how='any')
#2.Continent total trade
export_partner_continent=export_partner_composition_data_pd1.groupby(['RegionName'],as_index=False)[['TradeValue']].sum()
export_partner_continent['Country']=export_partner_continent['RegionName']
export_partner_continent['RegionName']='world'
#3.World total trade
export_partner_world=pd.DataFrame([export_partner_continent['TradeValue'].sum()],columns=['TradeValue'])
export_partner_world['Country']='world'
export_partner_world['RegionName']=""
#4.Append all dataframes
export_partner_composition_data_pd1=export_partner_composition_data_pd1.append(export_partner_continent[['Country','TradeValue','RegionName']]).append(export_partner_world[['Country','TradeValue','RegionName']])

#Deriving discrepancy indicators for each partner (Indicators are currently derived from a Tier 3 level)
median_di_data=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==3)).groupBy('CountryA').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd=median_di_data.toPandas()
median_di_data_pd=median_di_data_pd.rename(columns={'CountryA':'Country'})

viz_data=export_partner_composition_data_pd1.merge(median_di_data_pd,on='Country',how='left')
viz_data['TradeValue']=viz_data['TradeValue']/1000

fig = go.Figure(go.Treemap(labels=viz_data['Country'], values=viz_data['TradeValue'], parents=viz_data['RegionName'],customdata=viz_data['CIF_FOB_ratio'],textinfo = "label+percent entry" ,branchvalues='total',pathbar_textfont_size=10,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Overreporting export', '1','Underreporting export','2']
        ),cmax=2,cmin=0,cmid=1),hovertemplate='<b>Export destination: %{label}</b> <br><b>Region:</b> %{parent}<br><b>Export:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata}<extra></extra>'))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#uniformtext=dict(minsize=10,mode='hide'),
title_text='Senegal\'s export destinations (2021) and their CIF/FOB ratio'
fig.update_layout(
  
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text='Senegal\'s export destinations (2021) and their CIF/FOB ratio',font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 26

warnings.filterwarnings('ignore')


TIER=3

TRADE_FLOW=1

code_data=spark.sql('select * from prd_datascience_dianapublic.diana.productlist')
code_data=code_data.where((f.col('NomenclatureCode')=='HS') & (f.col('Tier')==TIER) )
code_data=code_data.select('ProductCode','ProductDescription').withColumn('ProductCodeL',f.substring('ProductCode',1,2)).join(product_group_hs_data.withColumn('ProductCodeR',f.col('ProductCode')).drop('ProductCode'),[f.col('ProductCodeL')==f.col('ProductCodeR')],how='full').select('ProductCode','ProductDescription','ProductGroup')

#Merging export composition data of country with the productDescription and ProductGroup
#ProductCode,Export,ProductDescription,ProductGroup
export_composition_data=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==3) & (f.col('CountryB').isin(country_list))).withColumn('Import',f.when(f.col('ImportAfromB')>f.col('ExportBToA'),f.col('ImportAfromB')).otherwise(f.col('ExportBToA'))).groupBy('ProductCode').agg(f.sum('Import').alias('Import')).join(code_data,'ProductCode',how='left').select('ProductCode','Import','ProductDescription','ProductGroup')

#Product Indicators data (used for coloring the map)
#median_di_data_1=discrepancy_data.where((f.col('CountryB')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==TIER) & (f.col('CountryA').isin(country_list)) & (f.length(f.col('ProductCode'))==2)).groupBy('ProductCode').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_1=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==TIER) & (f.col('CountryB').isin(country_list)) & (f.col('Tier')==TIER)).groupBy('ProductCode').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd_1=median_di_data_1.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Convert to pandas
export_composition_data_pd1=export_composition_data.toPandas()
#2.ProductGroup total trade
export_group=export_composition_data_pd1.groupby(['ProductGroup'],as_index=False)[['Import']].sum()
export_group['ProductCode']=export_group['ProductGroup']
export_group['ProductGroup']='All Commodities'
export_group['ProductDescription']=''
#3.All commodities total trade
export_group_all=pd.DataFrame([export_group['Import'].sum()],columns=['Import'])
export_group_all['ProductCode']='All Commodities'
export_group_all['ProductGroup']=""
export_group_all['ProductDescription']=''
#4.Append all dataframes
export_composition_data_pd1=export_composition_data_pd1.append(export_group[['ProductCode','Import','ProductDescription','ProductGroup']]).append(export_group_all[['ProductCode','Import','ProductDescription','ProductGroup']])
#export_composition_data_pd1


viz_data=export_composition_data_pd1.merge(median_di_data_pd_1,on='ProductCode',how='left')
commodity_names_pd=commodity_names.toPandas()
commodity_names_pd['CommodityCode']=commodity_names_pd['HS6 ID'].astype(str).str[-6:]
viz_data=viz_data.merge(commodity_names_pd[['CommodityCode','HS6']],left_on='ProductCode',right_on='CommodityCode',how='left').drop(columns={'CommodityCode'})
viz_data=viz_data.drop(columns='ProductDescription').rename(columns={'HS6':'ProductDescription'})
viz_data['Import']=viz_data['Import']/1000




fig = go.Figure(go.Treemap(labels=viz_data['ProductCode'], values=viz_data['Import'], parents=viz_data['ProductGroup'],customdata = viz_data[['ProductDescription','CIF_FOB_ratio']],textinfo = "label+percent entry", branchvalues='total',pathbar_textfont_size=15,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Negative transporation costs', '1','Positive transportation costs','2']
        ),cmin=0,cmax=2,cmid=1),hovertemplate='<b>Commodity Code: %{label}</b> <br><b>Commodity Description:</b>%{customdata[0]}<br><b>Commodity Group:</b> %{parent}<br><b>Import:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata[1]}<extra></extra>' ))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#hovertext=viz_data['CIF/FOB']  
#<br><b>X</b>: %{x}<br>'
text_title=COUNTRY+'\'s imports and their CIF/FOB ratio'

fig.update_layout(
uniformtext=dict(minsize=10,mode='hide'),
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text=text_title,font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 27
warnings.filterwarnings('ignore')
TRADE_FLOW=1
TIER=0



#Export partner data for countries
export_partner_composition_data=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==0) & (f.col('CountryB').isin(country_list))).withColumn('Import',f.when(f.col('ImportAfromB')>f.col('ExportBToA'),f.col('ImportAfromB')).otherwise(f.col('ExportBToA')))
export_partner_composition_data_pd=export_partner_composition_data.toPandas()

#Create the nested structure neccessary for the treemap viz
#1. Merge so region data
export_partner_composition_data_pd1=export_partner_composition_data_pd.rename(columns={'CountryB':'Country','Import':'TradeValue'})
export_partner_composition_data_pd1=export_partner_composition_data_pd1.merge(region_data_df,left_on='Country',right_on='ISO3',how='left')[['Country','TradeValue','RegionName']]
export_partner_composition_data_pd1=export_partner_composition_data_pd1.dropna(how='any')
#2.Continent total trade
export_partner_continent=export_partner_composition_data_pd1.groupby(['RegionName'],as_index=False)[['TradeValue']].sum()
export_partner_continent['Country']=export_partner_continent['RegionName']
export_partner_continent['RegionName']='world'
#3.World total trade
export_partner_world=pd.DataFrame([export_partner_continent['TradeValue'].sum()],columns=['TradeValue'])
export_partner_world['Country']='world'
export_partner_world['RegionName']=""
#4.Append all dataframes
export_partner_composition_data_pd1=export_partner_composition_data_pd1.append(export_partner_continent[['Country','TradeValue','RegionName']]).append(export_partner_world[['Country','TradeValue','RegionName']])

#Deriving discrepancy indicators for each partner (Indicators are currently derived from a Tier 3 level)
median_di_data=discrepancy_data.where((f.col('CountryA')==COUNTRY) & (f.col('Year')==REF_YEAR) & (f.col('Tier')==3)).groupBy('CountryB').agg(f.percentile_approx("DI", 0.5).alias("medianDI"),f.round(f.sum('ImportAfromB')/f.sum('ExportBtoA'),3).alias('CIF_FOB_ratio'))
median_di_data_pd=median_di_data.toPandas()
median_di_data_pd=median_di_data_pd.rename(columns={'CountryB':'Country'})

viz_data=export_partner_composition_data_pd1.merge(median_di_data_pd,on='Country',how='left')
viz_data['TradeValue']=viz_data['TradeValue']/1000


fig = go.Figure(go.Treemap(labels=viz_data['Country'], values=viz_data['TradeValue'], parents=viz_data['RegionName'],customdata=viz_data['CIF_FOB_ratio'],textinfo = "label+percent entry" ,branchvalues='total',pathbar_textfont_size=10,insidetextfont=dict(size=15),marker=dict(
        colors=viz_data['CIF_FOB_ratio'],
        colorscale='RdBu',showscale=True,colorbar=dict(
            title="CIF/FOB ratio",tickfont=dict(size=20),tickvals = [0, 0.5, 1,1.5,2],ticktext = ['0', 'Underreporting import', '1','Overreporting import','2']
        ),cmax=2,cmin=0,cmid=1),hovertemplate='<b>Import origin: %{label}</b> <br><b>Region:</b> %{parent}<br><b>Import:</b> %{value} Million $USD<br><b>CIF/FOB ratio:</b>%{customdata}<extra></extra>'))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#uniformtext=dict(minsize=10,mode='hide'),
text_title=COUNTRY+'\'s imports origins and their CIF/FOB ratio'
fig.update_layout(
  
    margin = dict(t=50, l=25, r=25, b=25),width=1400,
    height=1200,coloraxis_showscale=True,title=dict(text=text_title,font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
fig.show()

# COMMAND ----------

