# Databricks notebook source
import pandas as pd
import plotly.graph_objects as go
import pyspark.sql.functions as f
import plotly.express as px
import numpy as np


country_reporting_indicators_data=spark.sql('select * from prd_datascience_dianapublic.diana.country_trade_reporting_indicators ')
region_data=spark.sql("select * from prd_datascience_dianapublic.diana.countryregion")
region_data_df=region_data.toPandas()

# COMMAND ----------

# DBTITLE 1,Table 2
viz_table=country_reporting_indicators_data.where((f.col('Year')==2017) & (f.col('NomenCode')=='H0') & (f.col('flow_type')=='import_reporting' ) ).toPandas()

countries_order = ['CHN', 'GRC', 'IRL', 'MAR', 'BEL', 'KOR', 'MLT', 'BHS', 'GMB', 'LCA', 'TGO']
# Filter the DataFrame to include only the specified countries
filtered_df = viz_table[viz_table['Country'].isin(countries_order)]

# Convert the 'Country' column to a Categorical type with the specified order
filtered_df['Country'] = pd.Categorical(filtered_df['Country'], categories=countries_order, ordered=True)


#custom_column_names = ['<b>Country<b>', '<b>CIF/FOB Ratio<b>', '<b>Matched Median DI<b>', '<b>Matched CIF/FOB Ratio<b>', '<b>Missing Value Ratio<b>', '<b>Trade value of lost exports to total reported imports<b>']
custom_column_names = ['Country', 'CIF/FOB Ratio', 'Matched Median DI', 'Matched CIF/FOB Ratio', 'Missing Value Ratio', 'Trade value of lost exports to total reported imports']

# Assuming 'viz_table' is your original DataFrame
columns_to_select = ['Country', 'cif_fob_ratio', 'matched_median_DI', 'matched_cif_fob_ratio', 'unaccounted_value_ratio', 'lost_exports_to_reported_imports_ratio']
default_fill_color = 'white'  # Default color for most rows
gap_row_color = 'lightgrey'  # Color for the gap row
gap_row_index=6


# Filter the DataFrame and make sure to use .copy() to avoid SettingWithCopyWarning
filtered_df = viz_table[viz_table['Country'].isin(countries_order)].copy()
filtered_df['Country'] = pd.Categorical(filtered_df['Country'], categories=countries_order, ordered=True)
filtered_df = filtered_df.sort_values('Country')[columns_to_select].copy()


filtered_df[columns_to_select[1:]] = filtered_df[columns_to_select[1:]].round(3)
position_for_gap = countries_order.index('KOR') + 1  # Assuming 'KOR' is the country before the gap
gap_df = pd.DataFrame([[''] * len(filtered_df.columns)], columns=filtered_df.columns)

upper_df = filtered_df.iloc[:position_for_gap].copy()
lower_df = filtered_df.iloc[position_for_gap:].copy()
filtered_df = pd.concat([upper_df, gap_df, lower_df], ignore_index=True)

# Adjust colors to include the gap
fill_colors = [default_fill_color] * len(filtered_df)
fill_colors[position_for_gap] = gap_row_color  # Highlight the gap row

# Plotting
fig = go.Figure(data=[go.Table(
    header=dict(values=custom_column_names, fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=16, color='black')),
    cells=dict(values=[filtered_df[col].fillna('') for col in filtered_df.columns],
               fill_color=[fill_colors] * len(filtered_df.columns),
               align='center',line_color='black',font=dict(size=16, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=500,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 9
#Indicator map
import plotly.graph_objects as go
import pandas as pd

metric='weighted_DI'
FLOW_TYPE="export_reporting"
YEAR=2017
NOMEN_CODE='H0'


viz_data=country_reporting_indicators_data.where( (f.col('flow_type')==FLOW_TYPE) & (f.col('Year')==YEAR) & (f.col('NomenCode')==NOMEN_CODE) ).withColumn('TradeValue',f.greatest(f.col("reported_imports_sum"), f.col("reported_exports_sum"))).select('Country',metric,'TradeValue')

export_partner_composition_data_pd1=viz_data.toPandas()
export_partner_composition_data_pd1=export_partner_composition_data_pd1.merge(region_data_df,left_on='Country',right_on='ISO3',how='left')[['Country','TradeValue','RegionName']]
export_partner_composition_data_pd1=export_partner_composition_data_pd1.dropna(how='any')
#2.Continent total trade
export_partner_continent=export_partner_composition_data_pd1.groupby(['RegionName'],as_index=False)[['TradeValue']].sum()
export_partner_continent['Country']=export_partner_continent['RegionName']
export_partner_continent['RegionName']='world'
#3.World total trade
export_partner_world=pd.DataFrame([export_partner_continent['TradeValue'].sum()],columns=['TradeValue'])
export_partner_world['Country']='world'
export_partner_world['RegionName']=""
#4.Append all dataframes
#export_partner_composition_data_pd1=export_partner_composition_data_pd1.append(export_partner_continent[['Country','TradeValue','RegionName']]).append(export_partner_world[['Country','TradeValue','RegionName']])
export_partner_composition_data_pd1=pd.concat([export_partner_composition_data_pd1, export_partner_continent[['Country','TradeValue','RegionName']], export_partner_world[['Country','TradeValue','RegionName']]], ignore_index=True)


viz_data_map=export_partner_composition_data_pd1.merge(viz_data.select('Country',metric).toPandas(),on='Country',how='left')
#viz_data_map=viz_data_map.fillna(0)


fig = go.Figure(go.Treemap(labels=viz_data_map['Country'], values=viz_data_map['TradeValue'], parents=viz_data_map['RegionName'],textinfo = "label+percent entry" ,branchvalues='total',pathbar_textfont_size=10,insidetextfont=dict(size=22),marker=dict(
        colors=viz_data_map[metric],
        colorscale='RdBu_r',showscale=True,colorbar=dict(
            title="Discrepancy",tickfont=dict(size=22),tickvals = [-1, -0.5, 0, 0.5, 1],ticktext = ['-1', 'Underreporting export', '0','Overreporting export','1']
        ),cmax=1,cmin=-1,cmid=0), hovertext=viz_data_map[metric]))

fig.update_layout(margin = dict(t=50, l=25, r=25, b=25))

#uniformtext=dict(minsize=10,mode='hide'),
title=''
if FLOW_TYPE=='import_reporting':
  title+='Importers '
else:
  title+='Exporters '

title+='and their '+'weighted Discrepancy Index'

fig.update_layout(
  
    margin = dict(t=50, l=25, r=25, b=25),width=1800,
    height=1600,coloraxis_showscale=True,title=dict(text=title,font=dict(size=25))
)
#fig.update_layout(coloraxis_showscale=True)
print(viz_data_map)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 10
Year=2017
flow_code='import_reporting'
nomen_code='H0'

viz_data=country_reporting_indicators_data.where( (f.col('flow_type')==flow_code) & (f.col('Year')==Year) & (f.col('reported_imports_sum')>0) & (f.col('NomenCode')==nomen_code) & (f.col('Country')!='LSO')).toPandas()


# Create the scatter matrix
#'orphan_imports_to_reported_imports_ratio','percentage_overreported_transactions','percentage_underreported_transactions','median_DI','matched_median_DI','matched_std_DI',
cols=['cif_fob_ratio','matched_cif_fob_ratio','weighted_DI','matched_median_DI','percentage_lost_export','lost_exports_to_reported_imports_ratio','unaccounted_value_ratio']
labels={col: '<b>'+col.replace('_', ' ')+'</b>' for col in cols}
labels['lost_exports_to_reported_imports_ratio']='<b>lost exports to reported imports</b>'
fig = px.scatter_matrix(viz_data,
                        dimensions=cols,
                        labels=labels,
                        hover_data=['Country'])


#plot_bgcolor='white',
fig.update_layout( width=2200,  # Set the width of the figure
                  height=2150,
                  font=dict(size=15),margin=dict(l=0, r=0, t=100, b=100) ) # Adjust the font size here)
fig.update_traces(diagonal_visible=False, marker=dict(line=dict(width=1, color='DarkSlateGrey')))

# Update axes to be more visible
#fig.update_xaxes(showline=True, linewidth=2, linecolor='black', mirror=True)
#fig.update_yaxes(showline=True, linewidth=2, linecolor='black', mirror=True)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 11
import plotly.graph_objects as go
import pandas as pd

#Showing selected countries with extreme cif/fob ratios 
#This is the same plot as Figure 10, the first scatter plot in the last row of the scatter matrix
viz_data=country_reporting_indicators_data.where( (f.col('flow_type')==flow_code) & (f.col('Year')==Year) & (f.col('NomenCode')==nomen_code) & (f.col('Country')!='LSO')).toPandas()

viz_data['viz_11_label']=[name if b >=2 or b<0.33 else '' for name, b in zip(viz_data['Country'], viz_data['cif_fob_ratio'])]


fig = go.Figure(data=go.Scatter(
    x=viz_data['cif_fob_ratio'],
    y=viz_data['unaccounted_value_ratio'],
    mode='markers+text', 
    text=viz_data['viz_11_label'], 
    textposition='top center',  
    hoverinfo='text',
    marker=dict(size=9.5, color='#3457D5',line=dict(width=0.5, color='black')),
    textfont=dict(
        size=16,  
        color='black'  
    )
))

fig.update_layout(
    xaxis=dict(
        title='CIF/FOB ratio',
        title_font=dict(size=22),  
        tickfont=dict(size=16), 
    ),
    yaxis=dict(
        title='Missing value ratio',
        title_font=dict(size=22),  
        tickfont=dict(size=16),  
    ),
    width=1000,  
    height=600,   
)

fig.show()
